@@ -33,4 +33,23 @@
 #' Box, G.E., Jenkins, G.M., Reinsel, G.C. and Ljung, G.M. (2015) Time Series
 #' Analysis: Forecasting and Control. John Wiley & Sons, Hoboken.
 #' 
-"seriesJ"
\ No newline at end of file
+"seriesJ"
+
+#' Wisconsin Telephone Company
+#'
+#' Monthly data from January 1951 to October 1966.
+#'
+#' @format A object of class data.frame with 215 rows and 2 columns: 
+#' \describe{
+#'   \item{X}{Monthly outward station movements.} 
+#'   \item{Y}{Montly inward station movements.} 
+#' }
+#'
+#' @references 
+#' 
+#' Thompson, H. E. and Tiao, G. C. (1971) "Analysis of Telephone
+#' Data: A Case Study of Forecasting Seasonal Time Series," Bell Journal of
+#' Economics, The RAND Corporation, vol. 2(2), pages 515-541, Autumn.
+#'
+#' @source \url{https://drive.google.com/file/d/1LP8aMIQewMrxgOlrg9rN3eWHhZuUsY8K/view?usp=sharing}
+"Wtelephone"
\ No newline at end of file
